@@ -3,6 +3,7 @@ import matplotlib.gridspec as gridspec
 import numpy as np
 import seaborn as sns
 
+
 class Univariate:
     """
 
@@ -13,6 +14,8 @@ class Univariate:
     https://www3.nd.edu/~rwilliam/stats1/x03.pdf
     https://www.andrews.edu/~calkins/math/webtexts/statall.pdf
 
+    Usage:
+
     from argali import descriptive_statistics
     x = [1, 2, 3, 4, 3, 4, 5, 6, 7, 6, 7, 8, 7, 8, 8, 6, 5, 44, 3, 4, 5, 6, 7, 8, 9, 33, 22, 11, -1]
     x_summary = descriptive_statistics.Univariate(data=x)
@@ -24,6 +27,10 @@ class Univariate:
         self.data = data
 
     def mean(self):
+        """
+        calculates the mean of a single variable
+        :return: mean
+        """
         mean = sum(self.data) / len(self.data)
         return mean
 
@@ -222,8 +229,8 @@ class Univariate:
     def probability_density_function(self):
         sns.set(style="white", palette="muted", color_codes=True)
 
-        f, axes = plt.subplots(2, 2, figsize=(7, 7), sharex=True)
-        #sns.despine(left=True)
+        f, axes = plt.subplots(2, 2, figsize=(16, 10), sharex=True)
+        # sns.despine(left=True)
 
         # Plot a simple histogram with binsize determined automatically
         sns.distplot(self.data, kde=False, color="b", ax=axes[0, 0])
@@ -274,11 +281,43 @@ class Univariate:
         plt.ylabel('Frequency')
         plt.title("Histogram (Bins Determined By STURGES)")
 
-    def characteristic_function(self):
-        pass
+    def mean_plots(self):
+        fig2 = plt.figure(constrained_layout=True, figsize=(16, 10))
+        means = [self.mean(),
+                 self.harmonic_mean(),
+                 self.quadratic_mean(),
+                 self.geometric_mean(),
+                 self.trimmed_mean_01(),
+                 self.trimmed_mean_05(),
+                 self.trimmed_mean_10()]
 
-    def mean_absolute_deviation(self):
-        pass
+        means_plot = plt.bar(x=['Arithmetic',
+                                'Harmonic',
+                                'Quadratic',
+                                'Geometric',
+                                'Trimmed Mean 1%',
+                                'Trimmed Mean 5%',
+                                'Trimmed Mean 10%'], height=means)
+        plt.xlabel('Type of Mean')
+        plt.ylabel('Value')
+        plt.title("Comparison of Means")
+
+        return means_plot
+
+    def mean_vs_median(self):
+        fig2 = plt.figure(constrained_layout=True, figsize=(16, 10))
+        mvm = [self.mean(), self.median()]
+        mvm_plot = plt.bar(x=['Mean', 'Mode'], height=mvm)
+        plt.ylabel('Value')
+        plt.title("Mean Vs Median")
+
+        return mvm_plot
+
+    def violin_plot(self):
+        fig2 = plt.figure(constrained_layout=True, figsize=(16, 10))
+        violin_plot = plt.violinplot(self.data)
+
+        return violin_plot
 
     def unique_value_distribution(self):
         fig2 = plt.figure(constrained_layout=True, figsize=(16, 10))
@@ -348,10 +387,7 @@ class Univariate:
         x_summary.descriptive_summary()
         '''
         print("----------------- \n Descriptive Summary \n -----------------")
-        print("Number of Unique Values: ", self.unique_value_count(), "\n")
-
-        self.cumulative_distribution()
-
+        self.mean_plots()
         print("Arithmetic Mean: ")
         print("The mean is the only single number for which the residuals (deviations from the estimate) sum to "
               "zero. If it is required to use a single number as a typical value for a set of known numbers. The "
@@ -378,9 +414,6 @@ class Univariate:
               "is therefore less susceptible to sampling fluctuation than the mean for extremely skewed distributions. "
               "It is less efficient than the mean for normal distributions.")
 
-        self.histogram()
-        plt.show()
-
         print("\nTrimmed Mean (+- 1%): ", self.trimmed_mean_01())
         print("Trimmed Mean (+- 5%): ", self.trimmed_mean_05())
         print("Trimmed Mean (+- 10%): ", self.trimmed_mean_10())
@@ -388,13 +421,35 @@ class Univariate:
         print("\nMedian: \n The median is less sensitive to extreme scores than the mean and this makes it a better "
               "measure than the mean for highly skewed distributions. ", self.median())
 
-        self.unique_value_distribution()
-        plt.show()
-
         print("Mean, Median & Mode Comparison \n The mean, median, and mode are equal in symmetric distributions. The "
               "mean is typically higher than the median in positively skewed distributions and lower than the median "
               "in negatively skewed distributions.")
 
+        self.mean_vs_median()
+        plt.show()
+
+        print("\nSkew: \nA distribution is skewed if one of its tails is longer than the other. The skew of the "
+              "variable is: ", self.skew())
+        print("\nKurtosis: \nKurtosis is based on the size of a distribution's tails. Distributions with relatively "
+              "large tails are called leptokurtic; those with small tails are called platykurtic. A distribution with "
+              "the same kurtosis as the normal distribution is called mesokurtic. Variable kurtosis is: ",
+              self.kurtosis())
+
+        print("Frequency\nNumber of Unique Values: ", self.unique_value_count(), "\n")
+
+        print("The cumulative frequency distribution and related kernel density estimate is shown below:")
+        self.cumulative_distribution()
+        plt.show()
+
+        print("\nThe variance is computed as the average squared deviation of each number from its mean. The variance "
+              "for this variable is: ", self.variance())
+        self.probability_density_function()
+        plt.show()
+
+        print("A range of histograms with different bin calculations can be found below:")
+        self.histogram()
+        plt.show()
+
         print("\nRange: The range is the simplest measure of spread or dispersion: It is equal to the difference "
               "between the largest and the smallest values. The range of this variable is: ", self.range())
 
@@ -402,29 +457,35 @@ class Univariate:
         print("The value at the 70th percentile is: ", self.percentile_70())
         print("The interquartile range is ", self.interquartile_range())
 
-        print("\nThe variance is computed as the average squared deviation of each number from its mean. The variance "
-              "for this variable is: ", self.variance())
-
-        self.probability_density_function()
-        plt.show()
-
         print("\nThe standard deviation is the square root of the variance. This variable has a standard deviation of ",
               self.standard_deviation())
 
-        print("\nSkew: \nA distribution is skewed if one of its tails is longer than the other. The skew of the "
-              "variable is: ", self.skew())
-        print("\nKertosis: \nKurtosis is based on the size of a distribution's tails. Distributions with relatively "
-              "large tails are called leptokurtic; those with small tails are called platykurtic. A distribution with "
-              "the same kurtosis as the normal distribution is called mesokurtic. Variable kurtosis is: ",
-              self.kurtosis())
+        print(
+            "The below grid of plots includes additional frequency distributions based on unique values, a box plot and a squared difference plot ")
 
+        self.unique_value_distribution()
+        plt.show()
+        self.violin_plot()
         print("-----------------")
 
 
 class Bivariate:
-    pass
+    def __init__(self, variable_1, variable_2):
+        self.data_1 = variable_1
+        self.data_2 = variable_2
+
+    def comparison_mean(self):
+        pass
+
+    def comparison_median(self):
+        pass
+
+    def comparison_distribution(self):
+        pass
+
+    def comparison_box_plot(self):
+        pass
 
 
 class Multivariate:
     pass
-
